package com.ubertome.objview;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.res.AssetManager;
import android.content.res.Resources;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView;
import android.opengl.Matrix;
import android.preference.PreferenceManager;

import com.ubertome.objview.RenderFragment.RenderConfig;

public class RendererGL implements GLSurfaceView.Renderer{
	
	//because the renderer is running on a separate thread, 
	//must declare 'volatile' 
	public volatile float xAngle, yAngle, lightX, lightY = 4.0f,
						  xAccumAngle = 0, yAccumAngle = 0,
						  xLastAngle = 0, yLastAngle = 0;
	public volatile float pinchScaleFactor = 1.0f, pinchLightDist = 1.5f,
						  pinchAccumScale = 0, pinchLastScale = 0;
	public volatile float translationX = 0, translationY = 0;
	public volatile boolean enableLighting = true;
	public boolean modelLoaded = false;
	
	private int mProgramHandle = -1;
	private boolean lockToX = true, lockToY = true, lockToZ = true;
	
	static float red = 0.0f, green = 0.0f, blue = 0.0f, alpha = 1.0f;
	static float count = 1.0f;
	RenderConfig renderConfig;
	Mesh[] models;
	Context context;
	Resources res;
	AssetManager asm;
	ParsedObj obj = null;
	GShader vert, frag;
	ShaderProgram sp;
	Quaternion modelMatrixQLocal, newQ;
	float[] vec3;
	float ratio;
	public float fovy = 25;
	
	static float mModelMatrix[] = new float[16], mViewMatrix[] = new float[16], 
			mProjMatrix[] = new float[16], mMVPMatrix[] = new float[16], 
			mMVMatrix[] = new float[16], mNormalMatrix[] = new float[16];
	
//	public RendererGL(Resources resource, AssetManager AssetManager){
//		this.res = resource;
//		this.asm = AssetManager;
//		vert = new GShader(asm, "render.vert", GLES20.GL_VERTEX_SHADER);
//		frag = new GShader(asm, "render.frag", GLES20.GL_FRAGMENT_SHADER);
//		sp = new ShaderProgram(vert, frag);
//		mProgramHandle = sp.getHandle();
//	}
	
//	public RendererGL(AssetManager asm, ParsedObj pObj){
//		this.asm = asm;
//		obj = pObj;
//	}
	
	public RendererGL(Context context, RenderConfig configs){
		this.context = context;
		this.asm = context.getAssets();
		renderConfig = configs;
		vert = new GShader(asm, "shaders/render.vert", GLES20.GL_VERTEX_SHADER);
		frag = new GShader(asm, "shaders/render.frag", GLES20.GL_FRAGMENT_SHADER);
		sp = new ShaderProgram(vert, frag);
		Mesh.setShaderProgram(mProgramHandle = sp.getHandle());
		modelMatrixQLocal = new Quaternion();
		vec3 = new float[3];
	}

	@Override
	public void onSurfaceCreated(GL10 unused, EGLConfig config) {
		//sets background frame colour
//		GLES20.glClearColor(0.8f, 0.3f, 0.3f, 1.0f);
		GLES20.glClearColor(red, green, blue, alpha);
		
//		initShaders();
		
		vert = new GShader(asm, "shaders/render.vert", GLES20.GL_VERTEX_SHADER);
		frag = new GShader(asm, "shaders/render.frag", GLES20.GL_FRAGMENT_SHADER);
		sp = new ShaderProgram(vert, frag);
		Mesh.setShaderProgram(mProgramHandle = sp.getHandle());
		
		//initialize textures
		if(modelLoaded){
			for (int j = 0; j < models.length; j++) {
				models[j].loadTextures(context);
			}
		}
		

		
		GLES20.glEnable(GLES20.GL_DEPTH_TEST);
		GLES20.glDepthMask(true);
		GLES20.glDepthFunc(GLES20.GL_LEQUAL);
		GLES20.glDepthRangef(0.0f, 1.0f);
		
		SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(context);
		if(sharedPref.getBoolean("pref_culling_key", true))
			GLES20.glEnable(GLES20.GL_CULL_FACE);
		else
			GLES20.glDisable(GLES20.GL_CULL_FACE);
				
	}
	
	@Override
	public void onDrawFrame(GL10 unused){
		//Redraw background colour
		GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT | GLES20.GL_DEPTH_BUFFER_BIT);
		
		GLES20.glClearColor(renderConfig.bgBrightness,
				renderConfig.bgBrightness, renderConfig.bgBrightness, 1.0f);
		
		//Culling setting
		SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(context);
		if(sharedPref.getBoolean("pref_culling_key", true))
			GLES20.glEnable(GLES20.GL_CULL_FACE);
		else
			GLES20.glDisable(GLES20.GL_CULL_FACE);
		
		fovy = Float.valueOf(sharedPref.getString("pref_fovy_key", "25"));
		
		//Projection setting
		if (renderConfig.projectionChanged) {
			if (renderConfig.isOrthoView())
				Matrix.orthoM(mProjMatrix, 0, -ratio, ratio, -1f, 1f, 1.0f, 10f);
			else
				Matrix.perspectiveM(mProjMatrix, 0, fovy, ratio, 1.0f, 10.0f);
			renderConfig.projectionChanged = false;
		}
		
		if (modelLoaded) {
			for (int i = 0; i < models.length; i++) {
				float boundingRadius = models[i].getBoundingRadius();
				
				if(!models[i].texturesLoaded())
					models[i].loadTextures(context);
				
				xAccumAngle += xAngle - xLastAngle;
				yAccumAngle += yAngle - yLastAngle;
				
//				modelMatrixQLocal = models[i].modelMatrixQ;
//				newQ = new Quaternion();
				modelMatrixQLocal.buildFromEuler(-xAngle, -yAngle, 0);
				
//				modelMatrixQ.buildFromAxisAngle(
//						(lockToX) ? new float[]{1,0,0} : modelMatrixQ.x,
//						xAngle);
				
//				vec3[0] = 0; vec3[1] = 1; vec3[2] = 0;
//				newQ.buildFromAxis(vec3, xAngle);
//				modelMatrixQ.setEqualTo(newQ.multiplyWith(modelMatrixQ));
//				
//				vec3[0] = 1; vec3[1] = 0; vec3[2] = 0;
//				newQ.buildFromAxis(vec3, yAngle);
//				modelMatrixQLocal.setEqualTo(newQ.multiplyWith(modelMatrixQLocal));
				
				modelMatrixQLocal.normalize();
				
				xLastAngle = xAngle;
				yLastAngle = yAngle;
//				Matrix.setIdentityM(mModelMatrix, 0);
//				Matrix.rotateM(mModelMatrix, 0, xAngle, 0f, 1f, 0f);
//				Matrix.rotateM(mModelMatrix, 0, yAngle, 1f, 0f, 0f);
				
				mModelMatrix = modelMatrixQLocal.getMatrix();
				
				//Adjust mesh to the center of the world and scale to unit size
				Matrix.scaleM(mModelMatrix, 0, pinchScaleFactor,
						pinchScaleFactor, pinchScaleFactor);

				Matrix.scaleM(mModelMatrix, 0, 1.0f / boundingRadius,
						1.0f / boundingRadius, 1.0f / boundingRadius);
				
//				Matrix.multiplyMM(mNormalMatrix, 0, mViewMatrix, 0, mModelMatrix, 0);
				
				Matrix.translateM(mModelMatrix, 0,
						-models[i].getCenterPoint()[0],
						-models[i].getCenterPoint()[1],
						-models[i].getCenterPoint()[2]);
//				Matrix.translateM(mModelMatrix, 0, translationX, translationY, 0);	
				
				Matrix.setLookAtM(mViewMatrix, 0, 0, 0, 3, 0f, 0f, -1f, 0f, 1f, 0f);
				
				Matrix.multiplyMM(mMVMatrix, 0, mViewMatrix, 0, mModelMatrix, 0);
				Matrix.multiplyMM(mMVPMatrix, 0, mProjMatrix, 0, mMVMatrix, 0);
				
//				Matrix.invertM(mNormalMatrix, 0, mMVMatrix, 0);
//				Matrix.transposeM(mNormalMatrix, 0, mNormalMatrix, 0);
				
//				Matrix.setIdentityM(mModelMatrix, 0);
//				Matrix.rotateM(mModelMatrix, 0, 180, 1f, 0f, 0f);

				models[i].draw(mMVPMatrix, mMVMatrix, mModelMatrix, mViewMatrix,
						       renderConfig);
			}
		}

	}
	
	private void updateModelMatrix(float angleX, float angleY){
//		accumAngleY += angleY - 
	}
	
//	private void updateModelMatrix(){
//		xAccumAngle += xAngle - xLastAngle;
//		Matrix.rotateM(mModelMatrix, 0, xAccumAngle, 1, 0, 0);
//		xLastAngle = xAngle;
//		
//		yAccumAngle += yAngle - yLastAngle;
//		Matrix.rotateM(mModelMatrix, 0, yAccumAngle, 0, 1, 0);
//		yLastAngle = yAngle;
//		
//		pinchAccumScale = (pinchLastScale != 0) ? (pinchScaleFactor / pinchLastScale) : 1.0f;
//		Matrix.scaleM(mModelMatrix, 0, pinchAccumScale,
//				pinchAccumScale, pinchAccumScale);
//		pinchLastScale = pinchScaleFactor;
//	}
	
	@Override
	public void onSurfaceChanged(GL10 unused, int width, int height){
		GLES20.glViewport(0, 0, width, height);
		
		ratio = (float) width / height;
		
		SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(context);
		if(sharedPref.getBoolean("pref_culling_key", true))
			GLES20.glEnable(GLES20.GL_CULL_FACE);
		else
			GLES20.glDisable(GLES20.GL_CULL_FACE);
		fovy = Float.valueOf(sharedPref.getString("pref_fovy_key", "25"));
		
		if(renderConfig.isOrthoView())
			Matrix.orthoM(mProjMatrix, 0, -ratio, ratio, -1f, 1f, 1.0f, 10f);
		else
			Matrix.perspectiveM(mProjMatrix, 0, fovy, ratio, 1.0f, 10.0f);
	}
	

	public void loadModel(ParsedObj obj){
		this.obj = obj;
		try {
			// check that renderer has been initialized
			if (mProgramHandle != -1) {
				models = obj.getMeshes();
				for(int f = 0; f < models.length; f++){
					models[f].initBuffers();
//					models[f].loadTextures(context);
				}
				modelLoaded = true;
			} else {
				throw new ShaderErrorException(
						"Shaders have not been properly initialized! \r\nSource: RendererGL.loadModel()");
			}
		} catch(ShaderErrorException se){
			se.printStackTrace();
			Log.writeLog("Shaders have not been properly initialized! \r\nSource: RendererGL.loadModel()");
		}
	}
	

}
